@@ -5,6 +5,7 @@ __author__ = "giacomov"
 import collections
 import os
 import warnings
+from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 
 import numpy as np
 import pandas as pd
@@ -15,8 +16,8 @@ from astromodels.core.my_yaml import my_yaml
 from astromodels.core.parameter import IndependentVariable, Parameter
 from astromodels.core.tree import DuplicatedNode, Node
 from astromodels.functions.function import get_function
-from astromodels.sources.source import (EXTENDED_SOURCE, PARTICLE_SOURCE,
-                                        POINT_SOURCE, Source)
+from astromodels.sources import (ExtendedSource, ParticleSource, PointSource,
+                                 Source, SourceType)
 from astromodels.utils.disk_usage import disk_usage
 from astromodels.utils.logging import setup_logger
 from astromodels.utils.long_path_formatter import long_path_formatter
@@ -60,15 +61,17 @@ class Model(Node):
 
         # Dictionary to keep point sources
 
-        self._point_sources = collections.OrderedDict()
+        self._point_sources: Dict[str, PointSource] = collections.OrderedDict()
 
         # Dictionary to keep extended sources
 
-        self._extended_sources = collections.OrderedDict()
+        self._extended_sources: Dict[
+            str, ExtendedSource] = collections.OrderedDict()
 
         # Dictionary to keep particle sources
 
-        self._particle_sources = collections.OrderedDict()
+        self._particle_sources: Dict[
+            str, ParticleSource] = collections.OrderedDict()
 
         # Loop over the provided sources and process them
 
@@ -86,7 +89,9 @@ class Model(Node):
         # This will keep track of independent variables (if any)
         self._independent_variables = {}
 
-    def _add_source(self, source):
+    def _add_source(
+            self, source: Union[PointSource, ExtendedSource,
+                                ParticleSource]) -> None:
         """
         Remember to call _update_parameters after this!
         :param source:
@@ -101,10 +106,11 @@ class Model(Node):
 
             if isinstance(source, Source):
 
-                raise DuplicatedNode(
+                log.error(
                     "More than one source with the name '%s'. You cannot use the same name for multiple "
-                    "sources" % source.name
-                )
+                    "sources" % source.name)
+
+                raise DuplicatedNode()
 
             else:  # pragma: no cover
 
@@ -113,15 +119,15 @@ class Model(Node):
         # Now see if this is a point or extended source, and add them to the
         # appropriate dictionary
 
-        if source.source_type == POINT_SOURCE:
+        if source.source_type == SourceType.POINT_SOURCE:
 
             self._point_sources[source.name] = source
 
-        elif source.source_type == EXTENDED_SOURCE:
+        elif source.source_type == SourceType.EXTENDED_SOURCE:
 
             self._extended_sources[source.name] = source
 
-        elif source.source_type == PARTICLE_SOURCE:
+        elif source.source_type == SourceType.PARTICLE_SOURCE:
 
             self._particle_sources[source.name] = source
 
@@ -131,34 +137,36 @@ class Model(Node):
                 "Input sources must be either a point source or an extended source"
             )
 
-    def _remove_source(self, source_name):
+    def _remove_source(self, source_name: str):
         """
         Remember to call _update_parameters after this
         :param source_name:
         :return:
         """
 
-        assert source_name in self.sources, (
-            "Source %s is not part of the current model" % source_name
-        )
+        if not source_name in self.sources:
+
+            log.error(f"Source {source_name} is not part of the current model")
+
+            raise AssertionError()
 
         source = self.sources.pop(source_name)
 
-        if source.source_type == POINT_SOURCE:
+        if source.source_type == SourceType.POINT_SOURCE:
 
             self._point_sources.pop(source.name)
 
-        elif source.source_type == EXTENDED_SOURCE:
+        elif source.source_type == SourceType.EXTENDED_SOURCE:
 
             self._extended_sources.pop(source.name)
 
-        elif source.source_type == PARTICLE_SOURCE:
+        elif source.source_type == SourceType.PARTICLE_SOURCE:
 
             self._particle_sources.pop(source.name)
 
         self._remove_child(source_name)
 
-    def _find_parameters(self, node):
+    def _find_parameters(self, node) -> Dict[str, Parameter]:
 
         instances = collections.OrderedDict()
 
@@ -180,12 +188,12 @@ class Model(Node):
 
         return instances
 
-    def _update_parameters(self):
+    def _update_parameters(self) -> None:
 
-        self._parameters = self._find_parameters(self)
+        self._parameters: Dict[str, Parameter] = self._find_parameters(self)
 
     @property
-    def parameters(self):
+    def parameters(self) -> Dict[str, Parameter]:
         """
         Return a dictionary with all parameters
 
@@ -196,7 +204,7 @@ class Model(Node):
         return self._parameters
 
     @property
-    def free_parameters(self):
+    def free_parameters(self) -> Dict[str, Parameter]:
         """
         Get a dictionary with all the free parameters in this model
 
@@ -209,7 +217,8 @@ class Model(Node):
 
         # Filter selecting only free parameters
 
-        free_parameters_dictionary = collections.OrderedDict()
+        free_parameters_dictionary: Dict[
+            str, Parameter] = collections.OrderedDict()
 
         for parameter_name, parameter in list(self._parameters.items()):
 
@@ -220,7 +229,7 @@ class Model(Node):
         return free_parameters_dictionary
 
     @property
-    def linked_parameters(self):
+    def linked_parameters(self) -> Dict[str, Parameter]:
         """
         Get a dictionary with all parameters in this model in a linked status. A parameter is in a linked status
         if it is linked to another parameter (i.e. it is forced to have the same value of the other parameter), or
@@ -235,7 +244,8 @@ class Model(Node):
 
         # Filter selecting only free parameters
 
-        linked_parameter_dictionary = collections.OrderedDict()
+        linked_parameter_dictionary: Dict[
+            str, Parameter] = collections.OrderedDict()
 
         for parameter_name, parameter in list(self._parameters.items()):
 
@@ -245,7 +255,7 @@ class Model(Node):
 
         return linked_parameter_dictionary
 
-    def set_free_parameters(self, values):
+    def set_free_parameters(self, values: Iterable[float]) -> None:
         """
         Set the free parameters in the model to the provided values.
 
@@ -255,13 +265,20 @@ class Model(Node):
         :return: None
         """
 
-        assert len(values) == len(self.free_parameters)
+        if not len(values) == len(self.free_parameters):
 
-        for parameter, this_value in zip(list(self.free_parameters.values()), values):
+            log.error(
+                f"tried to pass {len(values)} parameters but need {len(self.free_parameters)}"
+            )
+
+            raise AssertionError()
+
+        for parameter, this_value in zip(list(self.free_parameters.values()),
+                                         values):
 
             parameter.value = this_value
 
-    def __getitem__(self, path):
+    def __getitem__(self, path: str):
         """
         Get a parameter from a path like "source_1.component.powerlaw.logK". This might be useful in certain
         context, although in an interactive analysis there is no reason to use this.
@@ -272,7 +289,7 @@ class Model(Node):
 
         return self._get_child_from_path(path)
 
-    def __contains__(self, path):
+    def __contains__(self, path: str) -> bool:
         """
         This allows the model to be used with the "in" operator, like;
 
@@ -314,7 +331,7 @@ class Model(Node):
             yield self.parameters[parameter]
 
     @property
-    def point_sources(self):
+    def point_sources(self) -> Dict[str, PointSource]:
         """
         Returns the dictionary of all defined point sources
 
@@ -323,7 +340,7 @@ class Model(Node):
         return self._point_sources
 
     @property
-    def extended_sources(self):
+    def extended_sources(self) -> Dict[str, ExtendedSource]:
         """
         Returns the dictionary of all defined extended sources
 
@@ -333,7 +350,7 @@ class Model(Node):
         return self._extended_sources
 
     @property
-    def particle_sources(self):
+    def particle_sources(self) -> Dict[str, ParticleSource]:
         """
         Returns the dictionary of all defined particle sources
 
@@ -343,7 +360,9 @@ class Model(Node):
         return self._particle_sources
 
     @property
-    def sources(self):
+    def sources(
+            self
+    ) -> Dict[str, Union[PointSource, ExtendedSource, ParticleSource]]:
         """
         Returns a dictionary containing all defined sources (of any kind)
 
@@ -351,15 +370,19 @@ class Model(Node):
 
         """
 
-        sources = collections.OrderedDict()
+        sources: Dict[str, Union[PointSource, ExtendedSource,
+                                 ParticleSource]] = collections.OrderedDict()
 
-        for d in (self.point_sources, self.extended_sources, self.particle_sources):
+        for d in (self.point_sources, self.extended_sources,
+                  self.particle_sources):
 
             sources.update(d)
 
         return sources
 
-    def add_source(self, new_source):
+    def add_source(
+        self, new_source: Union[PointSource, ExtendedSource,
+                                ParticleSource]) -> None:
         """
         Add the provided source to the model
 
@@ -371,21 +394,23 @@ class Model(Node):
 
         self._update_parameters()
 
-    def remove_source(self, source_name):
+    def remove_source(self, source_name: str) -> None:
         """
         Returns a new model with the provided source removed from the current model. Any parameters linked to the source to be removed are automatically unlinked.
 
         :param source_name: the name of the source to be removed
         :return: a new Model instance without the source
         """
-        
+
         self.unlink_all_from_source(source_name, warn=True)
 
         self._remove_source(source_name)
 
         self._update_parameters()
 
-    def unlink_all_from_source(self, source_name, warn=False):
+    def unlink_all_from_source(self,
+                               source_name: str,
+                               warn: bool = False) -> None:
         """
         Unlink all parameters of the current model that are linked to a parameter of a given source.
         To be called before removing a source from the model.
@@ -393,26 +418,28 @@ class Model(Node):
         :param source_name: the name of the source to which to remove all links
         :param warn: If True, prints a warning if any parameters were unlinked.
         """
-    
+
         tempmodel = Model(self[source_name])
         unlinked_parameters = collections.OrderedDict()
 
         for par in self.linked_parameters.values():
-        
-          target=par._aux_variable['variable']
 
-          if target.path in tempmodel:
+            target = par._aux_variable['variable']
+
+            if target.path in tempmodel:
+
+                unlinked_parameters[par.name] = par
+                self.unlink(par)
 
-              unlinked_parameters[par.name] = par
-              self.unlink(par)
-        
         if warn and unlinked_parameters:
-        
-            warnings.warn("The following %d parameters that were linked to source %s have been automatically un-linked: %s" %
-                          (len(unlinked_parameters), source_name, [p.path for p in unlinked_parameters.values() ] ), 
-                              RuntimeWarning)
 
-    def add_independent_variable(self, variable):
+            log.warning(
+                "The following %d parameters that were linked to source %s have been automatically un-linked: %s"
+                % (len(unlinked_parameters), source_name,
+                   [p.path for p in unlinked_parameters.values()]),
+                RuntimeWarning)
+
+    def add_independent_variable(self, variable: IndependentVariable) -> None:
         """
         Add a global independent variable to this model, such as time.
 
@@ -420,9 +447,10 @@ class Model(Node):
         :return: none
         """
 
-        assert isinstance(
-            variable, IndependentVariable
-        ), "Variable must be an instance of IndependentVariable"
+        if not isinstance(variable, IndependentVariable):
+            log.error("Variable must be an instance of IndependentVariable")
+
+            raise AssertionError()
 
         if self._has_child(variable.name):
 
@@ -433,7 +461,7 @@ class Model(Node):
         # Add also to the list of independent variables
         self._independent_variables[variable.name] = variable
 
-    def remove_independent_variable(self, variable_name):
+    def remove_independent_variable(self, variable_name: str) -> None:
         """
         Remove an independent variable which was added with add_independent_variable
 
@@ -446,7 +474,7 @@ class Model(Node):
         # Remove also from the list of independent variables
         self._independent_variables.pop(variable_name)
 
-    def add_external_parameter(self, parameter):
+    def add_external_parameter(self, parameter: Parameter) -> None:
         """
         Add a parameter that comes from something other than a function, to the model.
 
@@ -454,9 +482,11 @@ class Model(Node):
         :return: none
         """
 
-        assert isinstance(
-            parameter, Parameter
-        ), "Variable must be an instance of IndependentVariable"
+        if not isinstance(parameter, Parameter):
+
+            log.error("Variable must be an instance of IndependentVariable")
+
+            raise AssertionError()
 
         if self._has_child(parameter.name):
 
@@ -468,8 +498,7 @@ class Model(Node):
 
                 log.warning(
                     "External parameter %s already exist in the model. Overwriting it..."
-                    % parameter.name
-                )
+                    % parameter.name)
 
                 self._remove_child(parameter.name)
 
@@ -477,7 +506,7 @@ class Model(Node):
 
         self._add_child(parameter)
 
-    def remove_external_parameter(self, parameter_name):
+    def remove_external_parameter(self, parameter_name: str) -> None:
         """
         Remove an external parameter which was added with add_external_parameter
 
@@ -487,7 +516,7 @@ class Model(Node):
 
         self._remove_child(parameter_name)
 
-    def link(self, parameter_1, parameter_2, link_function=None):
+    def link(self, parameter_1, parameter_2, link_function=None) -> None:
         """
         Link the value of the provided parameters through the provided function (identity is the default, i.e.,
         parameter_1 = parameter_2).
@@ -506,14 +535,18 @@ class Model(Node):
             parameter_1_list = list(parameter_1)
 
         for param_1 in parameter_1_list:
-            assert param_1.path in self, (
-                "Parameter %s is not contained in this model" % param_1.path
-            )
+            if not param_1.path in self:
 
-        assert parameter_2.path in self, (
-            "Parameter %s is not contained in this model" % parameter_2.path
-        )
+                log.error( "Parameter %s is not contained in this model" % param_1.path )
 
+                raise AssertionError()
+                
+        if not parameter_2.path in self:
+
+            log.error( "Parameter %s is not contained in this model" % parameter_2.path )
+
+            raise AssertionError()
+            
         if link_function is None:
             # Use the Line function by default, with both parameters fixed so that the two
             # parameters to be linked will vary together
@@ -530,7 +563,7 @@ class Model(Node):
             # Now set the units of the link function
             link_function.set_units(parameter_2.unit, param_1.unit)
 
-    def unlink(self, parameter):
+    def unlink(self, parameter: Parameter) -> None:
         """
         Sets free one or more parameters which have been linked previously
 
@@ -560,7 +593,7 @@ class Model(Node):
                        
                     )
 
-    def display(self, complete=False):
+    def display(self, complete: bool=False) -> None:
         """
         Display information about the point source.
 
@@ -1020,7 +1053,7 @@ class Model(Node):
                     "report on the free space which follows: " % output_file,
                 )
 
-    def get_number_of_point_sources(self):
+    def get_number_of_point_sources(self) ->int:
         """
         Return the number of point sources
 
@@ -1028,7 +1061,7 @@ class Model(Node):
         """
         return len(self._point_sources)
 
-    def get_point_source_position(self, id):
+    def get_point_source_position(self, id) -> Tuple[float]:
         """
         Get the point source position (R.A., Dec)
 
@@ -1040,7 +1073,7 @@ class Model(Node):
 
         return pts.position.get_ra(), pts.position.get_dec()
 
-    def get_point_source_fluxes(self, id, energies, tag=None):
+    def get_point_source_fluxes(self, id: int, energies: np.ndarray, tag=None) -> np.ndarray:
         """
         Get the fluxes from the id-th point source
 
@@ -1053,14 +1086,14 @@ class Model(Node):
         set to a and the model evaluated in a.
         :return: fluxes
         """
-
+        
         return list(self._point_sources.values())[id](energies, tag=tag)
 
-    def get_point_source_name(self, id):
+    def get_point_source_name(self, id: int) -> str:
 
         return list(self._point_sources.values())[id].name
 
-    def get_number_of_extended_sources(self):
+    def get_number_of_extended_sources(self) -> int:
         """
         Return the number of extended sources
 
@@ -1068,7 +1101,7 @@ class Model(Node):
         """
         return len(self._extended_sources)
 
-    def get_extended_source_fluxes(self, id, j2000_ra, j2000_dec, energies):
+    def get_extended_source_fluxes(self, id: int, j2000_ra: float, j2000_dec: float, energies: np.ndarray) -> np.ndarray:
         """
         Get the flux of the id-th extended sources at the given position at the given energies
 
@@ -1081,7 +1114,7 @@ class Model(Node):
 
         return list(self._extended_sources.values())[id](j2000_ra, j2000_dec, energies)
 
-    def get_extended_source_name(self, id):
+    def get_extended_source_name(self, id: int) -> str:
         """
         Return the name of the n-th extended source
 
@@ -1091,7 +1124,7 @@ class Model(Node):
 
         return list(self._extended_sources.values())[id].name
 
-    def get_extended_source_boundaries(self, id):
+    def get_extended_source_boundaries(self, id: int):
 
         (ra_min, ra_max), (dec_min, dec_max) = list(self._extended_sources.values())[
             id
@@ -1099,7 +1132,7 @@ class Model(Node):
 
         return ra_min, ra_max, dec_min, dec_max
 
-    def is_inside_any_extended_source(self, j2000_ra, j2000_dec):
+    def is_inside_any_extended_source(self, j2000_ra: float, j2000_dec: float) -> bool:
 
         for ext_source in list(self.extended_sources.values()):
 
@@ -1119,7 +1152,7 @@ class Model(Node):
 
         return False
 
-    def get_number_of_particle_sources(self):
+    def get_number_of_particle_sources(self) -> int:
         """
         Return the number of particle sources
 
@@ -1128,7 +1161,7 @@ class Model(Node):
 
         return len(self._particle_sources)
 
-    def get_particle_source_fluxes(self, id, energies):
+    def get_particle_source_fluxes(self, id: int, energies: np.ndarray) -> np.ndarray:
         """
         Get the fluxes from the id-th point source
 
@@ -1139,11 +1172,11 @@ class Model(Node):
 
         return list(self._particle_sources.values())[id](energies)
 
-    def get_particle_source_name(self, id):
+    def get_particle_source_name(self, id: int) -> str:
 
         return list(self._particle_sources.values())[id].name
 
-    def get_total_flux(self, energies):
+    def get_total_flux(self, energies: np.ndarray) -> float:
         """
         Returns the total differential flux at the provided energies from all *point* sources
 
