@@ -4,14 +4,17 @@ import ast
 import collections
 import copy
 import inspect
+import math
 import os
 import re
 import sys
 import uuid
 from builtins import chr, map, str
 from operator import attrgetter
+from typing import Dict, List, Optional
 
 import astropy.units as u
+import numba as nb
 import numpy as np
 import six
 from yaml.reader import ReaderError
@@ -29,7 +32,7 @@ __author__ = 'giacomov'
 
 try:
 
-    from IPython.display import display, HTML
+    from IPython.display import HTML, display
 
 except:
 
@@ -489,7 +492,7 @@ class Function(Node):
 
     """
 
-    def __init__(self, name=None, function_definition=None, parameters=None):
+    def __init__(self, name: str=None, function_definition: str=None, parameters: Dict[str, Parameter]=None):
 
         # I use default values only to avoid warnings from pycharm and other software about the
         # calling sequence of this contructor. We actually need to enforce its proper use,
@@ -517,7 +520,7 @@ class Function(Node):
         # be different than the actual name of the parameter, use the .add_child method instead
         # of the add_children method
 
-        self._parameters = collections.OrderedDict()
+        self._parameters: Dict[str, Parameter] = collections.OrderedDict()
 
         for child_name, child in list(parameters.items()):
 
@@ -541,14 +544,14 @@ class Function(Node):
         self._is_prior = False
 
     @property
-    def n_dim(self):
+    def n_dim(self) -> int:
         """
         :return: number of dimensions for this function (1, 2 or 3)
         """
         return type(self)._n_dim
 
     @property
-    def free_parameters(self):
+    def free_parameters(self) -> Dict[str, Parameter]:
         """
         Returns a dictionary of free parameters for this function
 
@@ -561,7 +564,7 @@ class Function(Node):
         return free_parameters
 
     @property
-    def has_free_parameters(self):
+    def has_free_parameters(self) -> bool:
         """
         Returns True or False depending on if any parameters are free
         """
@@ -580,7 +583,7 @@ class Function(Node):
         """
         return uuid.UUID(bytes=os.urandom(16), version=4)
 
-    def has_fixed_units(self):
+    def has_fixed_units(self) -> bool:
         """
         Returns True if this function cannot change units, which is the case only for very specific functions (like
         models from foreign libraries like Xspec)
@@ -591,7 +594,7 @@ class Function(Node):
         return not (self._fixed_units is None)
 
     @property
-    def is_prior(self):
+    def is_prior(self) -> bool:
         """
         Returns False by default and must be overrided in the prior functions.
 
@@ -611,7 +614,7 @@ class Function(Node):
         return self._fixed_units
 
     @property
-    def description(self):
+    def description(self) -> str:
         """
         Returns a description for this function
         """
@@ -620,7 +623,7 @@ class Function(Node):
 
     # Add a property returning the parameters dictionary
     @property
-    def parameters(self):
+    def parameters(self) -> Dict[str, Parameter]:
         """
         Returns a dictionary of parameters
         """
@@ -913,7 +916,7 @@ class Function1D(Function):
         return self._x_unit
 
     @property
-    def y_unit(self):
+    def y_unit(self) -> u.Unit:
         """
         The unit of the dependent variable
         :return: a astropy.Unit instance
@@ -1019,7 +1022,7 @@ class Function1D(Function):
             return results
 
     @memoize
-    def fast_call(self, x):
+    def fast_call(self, x) -> np.ndarray:
 
         # Gather the current parameters' values without units, which means that the whole computation
         # will be without units, with a big speed gain (~10x)
@@ -1041,9 +1044,30 @@ class Function1D(Function):
         raise DesignViolation("Cannot call get_boundaries() on a 1d function")
 
 
+    def local_spectral_index(self, x, epsilon=1e-5):
+        """TODO describe function
+        
+        :param x: 
+        :type energy: 
+        :param epsilon: 
+        :type epsilon: 
+        :returns: 
+        
+        """
+        
+        a = self(x)
+        b = self(x * (1 + epsilon))
+
+        return _local_deriv(a, b, epsilon)
+    
+@nb.njit
+def _local_deriv(a, b,  epsilon):
+
+    return - np.log(a / b) / math.log( 1. + epsilon)
+    
 class Function2D(Function):
 
-    def __init__(self, name=None, function_definition=None, parameters=None):
+    def __init__(self, name: Optional[str] = None, function_definition: Optional[str]=None, parameters: Dict[str, Parameter] =None):
 
         Function.__init__(self, name, function_definition, parameters)
 
@@ -1661,7 +1685,8 @@ def get_function(function_name, composite_function_expression=None):
 
             # NOTE: import here to avoid circular import
 
-            from astromodels.functions.template_model import TemplateModel, MissingDataFile
+            from astromodels.functions.template_model import (MissingDataFile,
+                                                              TemplateModel)
 
             try:
 
