@@ -1,24 +1,26 @@
 from __future__ import division
 
 import collections
+import gc
 import os
 import re
-import warnings
 from builtins import object, range, str
+from dataclasses import dataclass
+from pathlib import Path
+from typing import Any, Dict, List, Optional, Union
 
 import astropy.io.fits as fits
 import astropy.units as u
+import h5py
 import numpy as np
-import pandas as pd
 import scipy.interpolate
 from future.utils import with_metaclass
-from pandas import HDFStore
-from pandas.api.types import infer_dtype
-from past.utils import old_div
+from interpolation import interp
+from interpolation.splines import eval_linear
 
 from astromodels.core.parameter import Parameter
 from astromodels.functions.function import Function1D, FunctionMeta
-from astromodels.utils.configuration import get_user_data_path
+from astromodels.utils import get_user_data_path
 from astromodels.utils.logging import setup_logger
 
 log = setup_logger(__name__)
@@ -27,7 +29,6 @@ log = setup_logger(__name__)
 # of the templates
 _TINY_ = 1e-50
 
-
 __all__ = [
     "IncompleteGrid",
     "ValuesNotInGrid",
@@ -53,41 +54,25 @@ class MissingDataFile(RuntimeError):
 _classes_cache = {}
 
 
-# currently python2 lacks support for the faster interpolations
-# thus we need to use some work arounds to keep the code compatible
-# with the different versions. This means template models are slower
-# in python2
+class GridInterpolate(object):
+    def __init__(self, grid, values):
+        self._grid = grid
+        self._values = np.ascontiguousarray(values)
 
-try:
+    def __call__(self, v):
 
-    from interpolation import interp
-    from interpolation.splines import eval_linear
-
-    class GridInterpolate(object):
-        def __init__(self, grid, values):
-            self._grid = grid
-            self._values = np.ascontiguousarray(values)
-
-        def __call__(self, v):
-
-            return eval_linear(self._grid, self._values, v)
-
-    class UnivariateSpline(object):
-        def __init__(self, x, y):
-
-            self._x = x
-            self._y = y
-
-        def __call__(self, v):
-
-            return interp(self._x, self._y, v)
+        return eval_linear(self._grid, self._values, v)
 
 
-except:
+class UnivariateSpline(object):
+    def __init__(self, x, y):
 
-    from scipy.interpolate import \
-        InterpolatedUnivariateSpline as UnivariateSpline
-    from scipy.interpolate import RegularGridInterpolator as GridInterpolate
+        self._x = x
+        self._y = y
+
+    def __call__(self, v):
+
+        return interp(self._x, self._y, v)
 
 
 class TemplateModelFactory(object):
@@ -110,8 +95,7 @@ class TemplateModelFactory(object):
         if re.match("[a-zA-Z_][a-zA-Z0-9_]*", name) is None:
             log.error(
                 "The provided name '%s' is not a valid name. You cannot use spaces, "
-                "or special characters"
-            )
+                "or special characters")
             raise RuntimeError()
 
         self._name = name
@@ -142,34 +126,48 @@ class TemplateModelFactory(object):
             self._parameters_grids[parameter_name] = None
 
         self._data_frame = None
-        self._multi_index = None
+
         self._interpolators = None
 
         self._interpolation_degree = interpolation_degree
 
         self._spline_smoothing_factor = int(spline_smoothing_factor)
 
-    def define_parameter_grid(self, parameter_name, grid):
+    def define_parameter_grid(self, parameter_name: str,
+                              grid: np.ndarray) -> None:
+        """
+        Define the parameter grid for this parameter.
+        Pass the name of the parameter and the array of values that it will take in the grid
+        """
 
-        assert parameter_name in self._parameters_grids, (
-            "Parameter %s is not part of this model" % parameter_name
-        )
+        if not parameter_name in self._parameters_grids:
+
+            log.error(f"Parameter {parameter_name} is not part of this model")
+
+            raise AssertionError()
 
         grid_ = np.array(grid)
 
-        assert (
-            grid_.shape[0] > 1
-        ), "A grid for a parameter must contain at least two elements"
+        if not (grid_.shape[0] > 1):
+
+            log.error(
+                "A grid for a parameter must contain at least two elements")
+
+            raise AssertionError()
 
         # Assert that elements are unique
 
-        assert np.all(np.unique(grid_) == grid_), (
-            "Non-unique elements in grid for parameter %s" % parameter_name
-        )
+        if not np.all(np.unique(grid_) == grid_):
+
+            log.error(
+                f"Non-unique elements in grid for parameter {parameter_name}")
+
+            raise AssertionError()
 
         self._parameters_grids[parameter_name] = grid_
 
-    def add_interpolation_data(self, differential_fluxes, **parameters_values_input):
+    def add_interpolation_data(self, differential_fluxes,
+                               **parameters_values_input):
 
         # Verify that the grid has been defined for all parameters
 
@@ -177,82 +175,117 @@ class TemplateModelFactory(object):
 
             if grid is None:
 
-                raise IncompleteGrid(
+                log.error(
                     "You need to define a grid for all parameters, by using the "
-                    "define_parameter_grid method."
-                )
+                    "define_parameter_grid method.")
+
+                raise IncompleteGrid()
+
+        # this is the first run and we will create
+        # a matrix that stores the data in an
+        # n_par,..., n_energies matrix
 
         if self._data_frame is None:
 
+            shape = []
+
+            for k, v in self._parameters_grids.items():
+
+                shape.append(len(v))
+
+            shape.append(self._energies.shape[0])
+
+            log.debug(f"grid shape: {shape}")
+
+            self._data_frame = np.zeros(tuple(shape))
+
+            log.debug(f"grid shape actual: {self._data_frame.shape}")
+
             # This is the first data set, create the data frame
 
             # Create the multi-index
 
-            self._multi_index = pd.MultiIndex.from_product(
-                list(self._parameters_grids.values()),
-                names=list(self._parameters_grids.keys()),
-            )
+            # self._multi_index = pd.MultiIndex.from_product(
+            #     list(self._parameters_grids.values()),
+            #     names=list(self._parameters_grids.keys()),
+            # )
 
-            # Pre-fill the data matrix with nans, so we will know if some elements have not been filled
+            # # Pre-fill the data matrix with nans, so we will know if some elements have not been filled
 
-            self._data_frame = pd.DataFrame(
-                index=self._multi_index, columns=self._energies
-            )
+            # self._data_frame = pd.DataFrame(index=self._multi_index,
+            #                                 columns=self._energies)
 
         # Make sure we have all parameters and order the values in the same way as the dictionary
-        parameters_values = np.zeros(len(self._parameters_grids)) * np.nan
 
-        for key in parameters_values_input:
+        parameter_idx = []
 
-            assert key in self._parameters_grids, "Parameter %s is not known" % key
+        for i, (k, v) in enumerate(self._parameters_grids.items()):
 
-            idx = list(self._parameters_grids.keys()).index(key)
+            if not k in parameters_values_input:
 
-            parameters_values[idx] = parameters_values_input[key]
+                log.error(f"Parameter {k} is not in input")
+
+                raise AssertionError()
+
+            parameter_idx.append(
+                int(np.where(v == parameters_values_input[k])[0][0]))
+
+        log.debug(f" have index {parameter_idx}")
 
         # If the user did not specify one of the parameters, then the parameters_values array will contain nan
 
-        assert np.all(
-            np.isfinite(parameters_values)
-        ), "You didn't specify all parameters' values."
+        if not len(parameter_idx) == len(self._parameters_grids):
+
+            log.error("You didn't specify all parameters' values.")
+
+            raise AssertionError()
 
         # Make sure we are dealing with pure numpy arrays (list and astropy.Quantity instances will be transformed)
         # First we transform the input into a u.Quantity (if it's not already)
 
         if not isinstance(differential_fluxes, u.Quantity):
 
-            differential_fluxes = (
-                np.array(differential_fluxes) * 1 / (u.keV * u.s * u.cm ** 2)
-            )  # type: u.Quantity
+            differential_fluxes = (np.array(differential_fluxes) * 1 /
+                                   (u.keV * u.s * u.cm**2))  # type: u.Quantity
 
         # Then we transform it in the right units and we cast it back to a pure np.array
 
         differential_fluxes = np.array(
-            differential_fluxes.to(old_div(1, (u.keV * u.s * u.cm ** 2))).value
-        )
+            differential_fluxes.to(1/ (u.keV * u.s * u.cm**2)).value)
 
         # Now let's check for valid inputs
 
-        assert self._energies.shape[0] == differential_fluxes.shape[0], (
-            "Differential fluxes and energies must have " "the same number of elements"
-        )
+        if not self._energies.shape[0] == differential_fluxes.shape[0]:
+
+            log.error("Differential fluxes and energies must have "
+                      "the same number of elements")
+
+            raise AssertionError()
 
         # Check that the provided value does not contains nan, inf nor zero (as the interpolation happens in the
         # log space)
-        assert np.all(
-            np.isfinite(differential_fluxes)
-        ), "You have invalid values in the differential flux (nan or inf)"
-        assert np.all(differential_fluxes >= 0), (
-            "You have negative values in the differential flux (which is of "
-            "course impossible)"
-        )
+        if not np.all(np.isfinite(differential_fluxes)):
+
+            log.error(
+                "You have invalid values in the differential flux (nan or inf)"
+            )
+
+            raise AssertionError()
+
+        if not np.all(differential_fluxes >= 0):
+
+            log.error(
+                "You have negative values in the differential flux (which is of "
+                "course impossible)")
+
+            raise AssertionError()
 
         if not np.all(differential_fluxes > 0):
 
             log.warning(
                 "You have zeros in the differential flux. Since the interpolation happens in the log space, "
-                "this cannot be accepted. We will substitute zeros with %g" % _TINY_
-            )
+                "this cannot be accepted. We will substitute zeros with %g" %
+                _TINY_)
 
             idx = differential_fluxes == 0  # type: np.ndarray
             differential_fluxes[idx] = _TINY_
@@ -261,60 +294,32 @@ class TemplateModelFactory(object):
 
         # Now set the values in the data frame
 
-        try:
+        self._data_frame[tuple(parameter_idx)] = np.atleast_2d(
+            differential_fluxes)
 
-            tmp = pd.to_numeric(differential_fluxes)
-
-            if len(parameters_values) == 1:
-
-                self._data_frame.loc[parameters_values.tolist()] = np.atleast_2d(tmp)
-
-            else:
-
-                self._data_frame.loc[tuple(parameters_values)] = tmp
-
-        except KeyError:
-
-            log.error(
-                "The provided parameter values (%s) are not in the defined grid"
-                % parameters_values
-            )
-
-            raise ValuesNotInGrid()
-
-    @staticmethod
-    def _clean_cols_for_hdf(data):
-
-        types = data.apply(lambda x: infer_dtype(x.values))
-
-        for col in types.index:
-
-            data[col] = pd.to_numeric(data[col])
-
-        return data
-
-    def save_data(self, overwrite=False):
+    def save_data(self, overwrite: bool = False):
 
         # First make sure that the whole data matrix has been filled
 
-        assert not self._data_frame.isnull().values.any(), (
-            "You have NaNs in the data matrix. Usually this means "
-            "that you didn't fill it up completely, or that some of "
-            "your data contains nans. Cannot save the file."
-        )
+        if np.any(np.isnan(self._data_frame)):
+
+            log.error("You have NaNs in the data matrix. Usually this means "
+                      "that you didn't fill it up completely, or that some of "
+                      "your data contains nans. Cannot save the file.")
+
+            raise AssertionError()
 
         # Get the data directory
 
-        data_dir_path = get_user_data_path()
+        data_dir_path: Path = get_user_data_path()
 
         # Sanitize the data file
 
-        filename_sanitized = os.path.abspath(
-            os.path.join(data_dir_path, "%s.h5" % self._name)
-        )
+        filename_sanitized: Path = data_dir_path / f"{self._name}.h5"
 
         # Check that it does not exists
-        if os.path.exists(filename_sanitized):
+
+        if filename_sanitized.exists():
 
             if overwrite:
 
@@ -326,8 +331,7 @@ class TemplateModelFactory(object):
 
                     log.error(
                         "The file %s already exists and cannot be removed (maybe you do not have "
-                        "permissions to do so?). " % filename_sanitized
-                    )
+                        "permissions to do so?). " % filename_sanitized)
 
                     raise IOError()
 
@@ -335,35 +339,25 @@ class TemplateModelFactory(object):
 
                 log.error(
                     "The file %s already exists! You cannot call two different "
-                    "template models with the same name" % filename_sanitized
-                )
+                    "template models with the same name" % filename_sanitized)
 
                 raise IOError()
 
         # Open the HDF5 file and write objects
 
-        with HDFStore(filename_sanitized) as store:
+        template_file: TemplateFile = TemplateFile(
+            name=self._name,
+            description=self._description,
+            spline_smoothing_factor=self._spline_smoothing_factor,
+            interpolation_degree=self._interpolation_degree,
+            grid=self._data_frame,
+            energies=self._energies,
+            parameters=self._parameters_grids,
+            parameter_order=list(self._parameters_grids.keys()))
 
-            # The _clean_cols_for_hdf is needed because for some reasons the format of some columns
-            # is not accepted by .to_hdf otherwise
-
-            self._clean_cols_for_hdf(self._data_frame).to_hdf(store, "data_frame")
-
-            store.get_storer("data_frame").attrs.metadata = {
-                "description": self._description,
-                "name": self._name,
-                "interpolation_degree": int(self._interpolation_degree),
-                "spline_smoothing_factor": self._spline_smoothing_factor,
-            }
-
-            for i, parameter_name in enumerate(self._parameters_grids.keys()):
-
-                store["p_%i_%s" % (i, parameter_name)] = pd.Series(
-                    self._parameters_grids[parameter_name]
-                )
-
-            store["energies"] = pd.Series(self._energies)
+        template_file.save(filename_sanitized)
 
+   
 
 # This adds a method to a class at runtime
 
@@ -383,12 +377,12 @@ class RectBivariateSplineWrapper(object):
     syntax as the other interpolation methods
 
     """
-
     def __init__(self, *args, **kwargs):
 
         # We can use interp2, which features spline interpolation instead of linear interpolation
 
-        self._interpolator = scipy.interpolate.RectBivariateSpline(*args, **kwargs)
+        self._interpolator = scipy.interpolate.RectBivariateSpline(
+            *args, **kwargs)
 
     def __call__(self, x):
 
@@ -397,6 +391,102 @@ class RectBivariateSplineWrapper(object):
         return res[0][0]
 
 
+@dataclass
+class TemplateFile:
+    """
+    simple container to read and write 
+    the data to an hdf5 file
+
+    """
+    name: str
+    description: str
+    grid: np.ndarray
+    parameters: Dict[str, np.ndarray]
+    parameter_order: List[str]
+    energies: np.ndarray
+    interpolation_degree: int
+    spline_smoothing_factor: float
+
+
+    def save(self, file_name: str):
+
+        """
+        serialize the contents to a file
+
+        :param file_name: 
+        :type file_name: str
+        :returns: 
+
+        """
+        with h5py.File(file_name, "w") as f:
+
+            f.attrs["name"] = self.name
+            f.attrs["description"] = self.description
+            f.attrs["interpolation_degree"] = self.interpolation_degree
+            f.attrs["spline_smoothing_factor"] = self.spline_smoothing_factor
+
+            f.create_dataset("energies", data=self.energies, compression="gzip")
+
+            f.create_dataset("grid", data=self.grid, compression="gzip")
+
+            # store the parameter order
+            dt = h5py.special_dtype(vlen=str)
+            po = np.array(self.parameter_order, dtype=dt) 
+            f.create_dataset('parameter_order', data=po)
+            par_group = f.create_group("parameters")
+            for k in self.parameter_order:
+
+                par_group.create_dataset(k, data=self.parameters[k], compression="gzip")
+
+    @classmethod
+    def from_file(cls, file_name: str):
+
+        """
+        read contents from a file
+
+        :param cls: 
+        :type cls: 
+        :param file_name: 
+        :type file_name: str
+        :returns: 
+
+        """
+        
+        with h5py.File(file_name, "r") as f:
+
+            name = f.attrs["name"]
+            description = f.attrs["description"] 
+            interpolation_degree = f.attrs["interpolation_degree"] 
+            spline_smoothing_factor = f.attrs["spline_smoothing_factor"] 
+
+            energies = f["energies"][()]
+            parameter_order = f["parameter_order"][()]
+
+            grid = f["grid"][()]
+
+            parameters = collections.OrderedDict()
+
+            for k in parameter_order:
+
+                parameters[k] = f["parameters"][k][()]
+            
+
+
+        return cls(name=name,
+                   description=description,
+                   interpolation_degree=interpolation_degree,
+                   spline_smoothing_factor=spline_smoothing_factor,
+                   energies=energies,
+                   parameter_order=parameter_order,
+                   parameters = parameters,
+                   grid=grid
+                   )
+
+                
+        
+
+
+    
 class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
     r"""
@@ -428,68 +518,64 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
         # Get the data directory
 
-        data_dir_path = get_user_data_path()
+        data_dir_path: Path = get_user_data_path()
 
         # Sanitize the data file
 
-        filename_sanitized = os.path.abspath(
-            os.path.join(data_dir_path, "%s.h5" % model_name)
-        )
+        filename_sanitized: Path = data_dir_path.absolute() /  f"{model_name}.h5"
+        
+        if not filename_sanitized.exists():
 
-        if not os.path.exists(filename_sanitized):
-
-            raise MissingDataFile(
-                "The data file %s does not exists. Did you use the "
-                "TemplateFactory?" % (filename_sanitized)
-            )
+            log.error("The data file %s does not exists. Did you use the "
+                "TemplateFactory?" % (filename_sanitized))
+            
+            raise MissingDataFile()
 
         # Open the template definition and read from it
 
         self._data_file = filename_sanitized
 
-        with HDFStore(filename_sanitized) as store:
+        # use the file shadow to read
+        
+        template_file: TemplateFile = TemplateFile.from_file(filename_sanitized)
 
-            self._data_frame = store["data_frame"]
+        self._parameters_grids = collections.OrderedDict()
 
-            self._parameters_grids = collections.OrderedDict()
+        
 
-            processed_parameters = 0
+        for key in template_file.parameter_order:
 
-            for key in list(store.keys()):
+            try:
 
-                match = re.search("p_([0-9]+)_(.+)", key)
+                # sometimes this is
+                # stored binary
+                
+                k = key.decode()
 
-                if match is None:
+            except(AttributeError):
 
-                    continue
+                # if not, then we
+                # load as a normal str
+                
+                k = key
+                                
+            log.debug(f"reading parameter {str(k)}")
 
-                else:
+  
+            self._parameters_grids[str(k)] = template_file.parameters[key]
 
-                    tokens = match.groups()
+            
 
-                    this_parameter_number = int(tokens[0])
-                    this_parameter_name = str(tokens[1])
+        self._energies = template_file.energies
 
-                    assert (
-                        this_parameter_number == processed_parameters
-                    ), "Parameters out of order!"
+        # Now get the metadata
 
-                    self._parameters_grids[this_parameter_name] = store[key]
+        description = template_file.description
+        name = template_file.name
 
-                    processed_parameters += 1
+        self._interpolation_degree = template_file.interpolation_degree
 
-            self._energies = np.array(store["energies"])
-
-            # Now get the metadata
-
-            metadata = store.get_storer("data_frame").attrs.metadata
-
-            description = metadata["description"]
-            name = metadata["name"]
-
-            self._interpolation_degree = metadata["interpolation_degree"]
-
-            self._spline_smoothing_factor = metadata["spline_smoothing_factor"]
+        self._spline_smoothing_factor = template_file.spline_smoothing_factor
 
         # Make the dictionary of parameters
 
@@ -499,6 +585,7 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
         function_definition["latex"] = "n.a."
 
+        
         # Now build the parameters according to the content of the parameter grid
 
         parameters = collections.OrderedDict()
@@ -512,7 +599,7 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
             parameters[parameter_name] = Parameter(
                 parameter_name,
-                grid.median(),
+                np.median(grid),
                 min_value=grid.min(),
                 max_value=grid.max(),
             )
@@ -529,16 +616,22 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
         # Finally prepare the interpolators
 
-        self._prepare_interpolators(log_interp)
+        self._prepare_interpolators(log_interp, template_file.grid)
 
-    def _prepare_interpolators(self, log_interp):
+        # try can clean up the file
+        
+        del template_file
+
+        gc.collect()
+                
+    def _prepare_interpolators(self, log_interp, data_frame):
 
         # Figure out the shape of the data matrices
         data_shape = [x.shape[0] for x in list(self._parameters_grids.values())]
 
         self._interpolators = []
 
-        for energy in self._energies:
+        for i, energy in enumerate(self._energies):
 
             # Make interpolator for this energy
             # NOTE: we interpolate on the logarithm
@@ -547,7 +640,7 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
             if log_interp:
 
                 this_data = np.array(
-                    np.log10(self._data_frame[energy].values).reshape(*data_shape),
+                    np.log10(data_frame[...,i]).reshape(*data_shape),
                     dtype=float,
                 )
 
@@ -557,11 +650,12 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
                 # work in linear space
                 this_data = np.array(
-                    self._data_frame[energy].values.reshape(*data_shape), dtype=float
+                    data_frame[..., i].reshape(*data_shape), dtype=float
                 )
 
                 self._is_log10 = False
 
+                
             if len(list(self._parameters_grids.values())) == 2:
 
                 x, y = list(self._parameters_grids.values())
@@ -621,11 +715,14 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
             self._interpolators.append(this_interpolator)
 
+        # clear the data
+        self._data_frame = None
+            
     def _set_units(self, x_unit, y_unit):
 
         self.K.unit = y_unit
 
-        self.scale.unit = old_div(1, x_unit)
+        self.scale.unit = 1 / x_unit
 
     # This function will be substituted during construction by another version with
     # all the parameters of this template
@@ -648,7 +745,7 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
             # Same for the scale
 
-            scale = scale.to(old_div(1, u.keV)).value
+            scale = scale.to(1/ u.keV).value
 
         if self._is_log10:
 
@@ -694,8 +791,29 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
         # NOTE: the units are added back through the multiplication by K in the evaluate method
 
-        return old_div(values, scale)
+        return values / scale
 
+    def clean(self):
+        """
+        Table models can consume a lot of memory. If are creating lots of 
+        table models in memory for simulations, you may want to call
+        clean on the model try and remove some of the memory consumed by the models
+
+        :returns: 
+
+        """
+        
+        self._interpolators = None
+        del self._interpolators
+        gc.collect()
+
+        log.info("You have 'cleaned' the table model at it will no longer be useable")
+
+    def __del__(self):
+
+        self.clean()
+
+        
     @property
     def data_file(self):
 
@@ -834,3 +952,109 @@ class XSPECTableModel(object):
             tmf.add_interpolation_data(self._spectrum[i, :], **input_dict)
 
         tmf.save_data(overwrite=overwrite)
+
+
+
+def convert_old_table_model(model_name: str):
+    from pandas import HDFStore
+
+    # Get the data directory
+
+    data_dir_path: Path = get_user_data_path()
+
+    # Sanitize the data file
+    
+    filename_sanitized = data_dir_path / f"{model_name}.h5"
+    
+    
+    if not filename_sanitized.exists():
+
+        log.error("The data file %s does not exists. Did you use the "
+             "TemplateFactory?" % (filename_sanitized))
+         
+        raise MissingDataFile( )
+    
+    with HDFStore(filename_sanitized) as store:
+    
+        data_frame = store["data_frame"]
+
+        parameters_grids = collections.OrderedDict()
+
+        processed_parameters = 0
+
+        for key in list(store.keys()):
+
+            match = re.search("p_([0-9]+)_(.+)", key)
+            
+            if match is None:
+                
+                continue
+            
+            else:
+                
+                tokens = match.groups()
+                
+                this_parameter_number = int(tokens[0])
+                this_parameter_name = str(tokens[1])
+                
+                
+                if not this_parameter_number == processed_parameters:
+
+                    log.error("Parameters out of order!")
+
+                    raise AssertionError()
+                
+                parameters_grids[this_parameter_name] = store[key]
+                
+                processed_parameters += 1
+                
+        energies = np.array(store["energies"])
+
+        shape = [len(x) for _, x in parameters_grids.items()]
+
+        shape.append(len(energies))
+
+        grid = np.zeros(shape)
+
+        for i, e in enumerate(energies):
+
+            data = data_frame[e].values.reshape(*shape[:-1])
+
+            grid[...,i] = data
+        
+                     
+        
+        # Now get the metadata
+            
+        metadata = store.get_storer("data_frame").attrs.metadata
+        
+        description = metadata["description"]
+        
+        name = metadata["name"]
+        
+        interpolation_degree = metadata["interpolation_degree"]
+        
+        spline_smoothing_factor = metadata["spline_smoothing_factor"]
+        
+        # Make the dictionary of parameters
+
+        function_definition = collections.OrderedDict()
+
+        function_definition["description"] = description
+
+
+
+    template_file: TemplateFile = TemplateFile(
+        name=name,
+        description=description,
+        spline_smoothing_factor=spline_smoothing_factor,
+        interpolation_degree=interpolation_degree,
+        grid=grid,
+        energies=energies,
+        parameters=parameters_grids,
+        parameter_order=list(parameters_grids.keys()))
+
+    template_file.save(filename_sanitized)
+
+
+        
