@@ -6,14 +6,19 @@ import numpy as np
 from astromodels.core.spectral_component import SpectralComponent
 from astromodels.core.tree import Node
 from astromodels.core.units import get_units
-from astromodels.functions.functions import Constant
-from astromodels.sources.source import Source, EXTENDED_SOURCE
+from astromodels.functions import Constant
+from astromodels.sources.source import Source, SourceType
 from astromodels.utils.pretty_list import dict_to_list
+from astromodels.utils.logging import setup_logger
 
+log = setup_logger(__name__)
 
 class ExtendedSource(Source, Node):
-
-    def __init__(self, source_name, spatial_shape, spectral_shape=None, components=None):
+    def __init__(self,
+                 source_name,
+                 spatial_shape,
+                 spectral_shape=None,
+                 components=None):
 
         # Check that we have all the required information
         # and set the units
@@ -39,7 +44,8 @@ class ExtendedSource(Source, Node):
 
             # Components in this case have energy as x and differential flux as y
 
-            diff_flux_units = (current_u.energy * current_u.area * current_u.time) ** (-1)
+            diff_flux_units = (current_u.energy * current_u.area *
+                               current_u.time)**(-1)
 
             # Now set the units of the components
             for component in components:
@@ -47,7 +53,8 @@ class ExtendedSource(Source, Node):
                 component.shape.set_units(current_u.energy, diff_flux_units)
 
             # Set the units of the brightness
-            spatial_shape.set_units(current_u.angle, current_u.angle, current_u.angle**(-2))
+            spatial_shape.set_units(current_u.angle, current_u.angle,
+                                    current_u.angle**(-2))
 
         elif spatial_shape.n_dim == 3:
 
@@ -62,19 +69,24 @@ class ExtendedSource(Source, Node):
                 components = [SpectralComponent("main", spectral_shape)]
 
                 # set the units
-                diff_flux_units = (current_u.energy * current_u.area * current_u.time *
-                                   current_u.angle**2) ** (-1)
-                spatial_shape.set_units(current_u.angle, current_u.angle, current_u.energy, diff_flux_units)
+                diff_flux_units = (current_u.energy * current_u.area *
+                                   current_u.time * current_u.angle**2)**(-1)
+                spatial_shape.set_units(current_u.angle, current_u.angle,
+                                        current_u.energy, diff_flux_units)
 
             else:
 
                 # the spectral shape has been given, so this is a case where the spatial template gives an
                 # energy-dependent shape and the spectral components give the spectrum
 
-                assert (spectral_shape is not None) ^ (components is not None), "You can provide either a single " \
-                                                                                "component, or a list of components " \
-                                                                                "(but not both)."
+                if not ((spectral_shape is not None) ^ (components is not None)):
 
+                    log.error("You can provide either a single "
+                              "component, or a list of components "
+                              "(but not both).")
+
+                    raise AssertionError()
+                    
                 if spectral_shape is not None:
 
                     components = [SpectralComponent("main", spectral_shape)]
@@ -91,11 +103,13 @@ class ExtendedSource(Source, Node):
 
         else:
 
-            raise RuntimeError("The spatial shape must have either 2 or 3 dimensions.")
+            log.error("The spatial shape must have either 2 or 3 dimensions.")
+
+            raise RuntimeError()
 
         # Here we have a list of components
 
-        Source.__init__(self, components, EXTENDED_SOURCE)
+        Source.__init__(self, components, SourceType.EXTENDED_SOURCE)
 
         # A source is also a Node in the tree
 
