@@ -2,6 +2,7 @@ from __future__ import print_function
 from __future__ import division
 import astropy.units as u
 import numpy as np
+import numpy.testing as npt
 import pytest
 
 from astromodels.core.spectral_component import SpectralComponent
@@ -37,7 +38,7 @@ else:
 
 try:
 
-    from astromodels.functions.functions import EBLattenuation
+    from astromodels.functions import EBLattenuation
 
 except:
 
@@ -200,9 +201,7 @@ def test_call_with_units():
             else:
 
                 x_unit_to_use = u.keV
-
-
-
+                
             # Use the function as a spectrum
             ps = PointSource("test", 0, 0, instance)
 
@@ -393,3 +392,15 @@ def test_free_param():
         param.free = True
         assert len(source.free_parameters) == i+1
 
+
+
+
+    def test_local_deriv():
+
+        p = Powerlaw(index=-2.)
+
+
+        npt.assert_allclose(-2., p.local_spectral_index(np.logspace(1,3,10)))
+        
+
+        
