@@ -9,7 +9,7 @@ import warnings
 
 from astromodels.core.my_yaml import my_yaml
 from astromodels.functions.function import get_function_class
-from astromodels.utils.configuration import get_user_data_path
+from astromodels.utils import get_user_data_path
 
 
 class XSpecNotAvailable(ImportWarning):
@@ -530,7 +530,8 @@ $DOCSTRING$
     def _set_units(self, x_unit, y_unit):
 
         # Make sure this is an energy
-        assert str(x_unit.physical_type) == 'energy', "Trying to set x-unit with (%s), which is not energy" % (x_unit)
+        #assert str(x_unit.physical_type) == 'energy', "Trying to set x-unit with (%s), which is not energy" % (x_unit)
+        assert 'energy' in str(x_unit.physical_type), "Trying to set x-unit with (%s), which is not energy" % (x_unit)
 
         # Make sure the y_unit is the correct one
         try:
